/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include "execution_defs.h"
#include "common/common.h"
#include "index/ix.h"
#include "system/sm.h"

class AbstractExecutor {
   public:
    Rid _abstract_rid;

    Context *context_;

    virtual ~AbstractExecutor() = default;

    virtual size_t tupleLen() const { return 0; };

    virtual const std::vector<ColMeta> &cols() const {
        std::vector<ColMeta> *_cols = nullptr;
        return *_cols;
    };

    virtual std::string getType() { return "AbstractExecutor"; };

    virtual void beginTuple(){};

    virtual void nextTuple(){};

    virtual bool is_end() const { return true; };

    virtual Rid &rid() = 0;

    virtual std::unique_ptr<RmRecord> Next() = 0;

    virtual ColMeta get_col_offset(const TabCol &target) { return ColMeta();};

    std::vector<ColMeta>::const_iterator get_col(const std::vector<ColMeta> &rec_cols, const TabCol &target) {
        auto pos = std::find_if(rec_cols.begin(), rec_cols.end(), [&](const ColMeta &col) {
            return col.tab_name == target.tab_name && col.name == target.col_name;
        });
        if (pos == rec_cols.end()) {
            throw ColumnNotFoundError(target.tab_name + '.' + target.col_name);
        }
        return pos;
    }

    std::vector<ColMeta>::const_iterator get_col(const std::vector<ColMeta> &rec_cols, const ColMeta &target) {
        auto pos = std::find_if(rec_cols.begin(), rec_cols.end(), [&](const ColMeta &col) {
            return col.tab_name == target.tab_name && col.name == target.name;
        });
        if (pos == rec_cols.end()) {
            //assert(0);
            throw ColumnNotFoundError(target.tab_name + '.' + target.name);
        }
        return pos;
    }

    /**
     *
     * @param record,行记录
     * @param col ，需要取的数据的列
     * @return 返回一个Value
     */
    Value get_value(const std::unique_ptr<RmRecord> &record, const ColMeta& col) const {
        char *data = record->data + col.offset;
        size_t len = col.len;
        Value result;
        result.type = col.type;
        if(col.type == TYPE_INT) {
            int tmp;
            memcpy((char*)&tmp, data, len);
            result.set_int(tmp);
        }else if(col.type == TYPE_FLOAT) {
            float tmp;
            memcpy((char*)&tmp, data, len);
            result.set_float(tmp);
        }else if(col.type == TYPE_STRING) {
            std::string tmp(data, len);
            result.set_str(tmp);
        } else{
            assert(0);
        }
        result.init_raw(len);
        return result;
    }

};