//
// Created by ll on 7/5/24.
//

#include "record/rm_defs.h"
#include <memory>
#include "common/config.h"

class JoinBlock {  // 连接块缓冲池
public:

    explicit JoinBlock(int blockSize = 10)
            : block_size(blockSize), pos(0), len(0) {
        records.resize(block_size + 5);
    }
    void push(std::unique_ptr<RmRecord> r) {
        records[len++] = std::move(r);
    }
    RmRecord* getRecord(){
        if (records[pos]) {
            return records[pos++].get();
        }
    }

    bool isFull() {
        return len >= JOINBLOCK_SIZE;
    }
    bool isEnd() {
        return pos >= len;
    }

    int getPos() const {
        return pos;
    }

    void setPos(int pos) {
        JoinBlock::pos = pos;
    }

    void addOnePos(){
        JoinBlock::pos = getPos() + 1;
    }

    int getLen() const {
        return len;
    }
    void reset() {
        for (auto& record : records) {
            record.reset();
        }
        pos = 0;
        len = 0;
    }

    int pos;         // 当前读取位置
    int len;
private:
    int block_size;  // 缓冲池的块大小
   // int contain_len;         // 当前记录数量
    std::vector<std::unique_ptr<RmRecord>> records;  // 记录缓冲区
};
