/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include <map>
#include <unordered_map>
#include "log_manager.h"
#include "storage/disk_manager.h"
#include "system/sm_manager.h"

class RedoLogsInPage {
public:
    RedoLogsInPage() { table_file_ = nullptr; }
    RedoLogsInPage(RmFileHandle *table_file){table_file_ = table_file;}
    RmFileHandle* table_file_;
    std::vector<lsn_t> redo_logs_;   // 在该page上需要redo的操作的lsn//疑问，如果logs很多，
};

class RecoveryManager {
public:
    RecoveryManager(DiskManager* disk_manager, BufferPoolManager* buffer_pool_manager, SmManager* sm_manager,LogManager *log_manager) {
        disk_manager_ = disk_manager;
        buffer_pool_manager_ = buffer_pool_manager;
        sm_manager_ = sm_manager;
        log_manager_ = log_manager;
    }


    void analyze();
    void redo();
    void redoInsertLog(InsertLogRecord *rec,PageId page_id,RmFileHandle* file_handle);
    void redoDeleteLog(DeleteLogRecord *rec,PageId page_id,RmFileHandle* file_handle);
    void redoUpdateLog(UpdateLogRecord *rec,PageId page_id,RmFileHandle* file_handle);
    void allocPage(Rid& rid, RmFileHandle* fh_);

    void undo();
    void undoInsertLog(InsertLogRecord *rec);
    void undoDeleteLog(DeleteLogRecord *rec);
    void undoUpdateLog(UpdateLogRecord *rec);

    void maintain_index_consistency();



private:

    LogRecord *generator_type2log(LogType type){
        switch (type)
        {
            case LogType::begin :
                return new BeginLogRecord();
            case LogType::ABORT :
                return new AbortLogRecord();
            case LogType::commit :
                return new CommitLogRecord();
            case LogType::INSERT :
                return new InsertLogRecord();
            case LogType::UPDATE :
                return new UpdateLogRecord();
            case LogType::DELETE :
                return new DeleteLogRecord();
            default:
                assert(0);
                break;
        }

    }
    void dml_update_dpt_map(LogRecord *rec);

    //utility
    //
    LogRecord* getRecByLsn(lsn_t lsn){
        auto [offset,total_len] = lsn_map_[lsn];
        char *single_log_buffer = new char[total_len];
        disk_manager_->read_log(single_log_buffer,total_len,offset);
        //assert(disk_manager_->read_log(single_log_buffer,total_len,offset));
        auto log_type = *reinterpret_cast<const LogType *>(single_log_buffer + OFFSET_LOG_TYPE);
        auto rec = generator_type2log(log_type);
        rec->deserialize(single_log_buffer);
        return rec;
    }

    LogBuffer buffer_;                                              // 读入日志
    DiskManager* disk_manager_;                                     // 用来读写文件
    BufferPoolManager* buffer_pool_manager_;                        // 对页面进行读写
    SmManager* sm_manager_;                                         // 访问数据库元数据


    //自己加
    LogManager * log_manager_;
    std::unordered_map<txn_id_t ,lsn_t > undo_txn_map_;
    std::unordered_map<txn_id_t ,lsn_t >redo_txn_map_;
    std::unordered_map<PageId,RedoLogsInPage> dpt_map_;
    std::unordered_map<lsn_t ,std::pair<int,uint32_t> >lsn_map_;                         //记载每一log在disk内的偏移量和size
    std::vector<lsn_t> redo_list;
    std::vector<lsn_t> read_list;
    std::unordered_map<txn_id_t, lsn_t> active_txn_;
    void UndoLog(LogRecord *log_record);
    void RedoLog(LogRecord *log_record);
};